// Copyright 2011 Thomas Kamps anubis1@linux-ecke.de
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#include "lightweight.h"
#include "ui_lightweight.h"

#include <QFile>
#include <QDir>
#include <QShortcut>
#include <QCryptographicHash>
#include <QSettings>

#include <QDebug>

Lightweight::Lightweight(QWidget *parent) : QWidget(parent), ui(new Ui::Lightweight) {
	ui->setupUi(this);

	tabs = new QStackedLayout();
	tabList = new QList<BrowserTab*>();
	ui->verticalLayout->addLayout(tabs);

	ui->verticalLayout->setSpacing(0);
	currentTab = 0;
	fullscreen = false;

	//Create bookmark
	bookmarkMenu = new QMenu();
	Bookmark* acnewTab = new Bookmark(QIcon(":/icons/new.png"), "about:blank", tr("Empty page"));
	acnewTab->hideDeleteButton();
	bookmarkMenu->addAction(acnewTab);
	bookmarkMenu->addSeparator();
	ui->bNewTab->setMenu(bookmarkMenu);
	ui->bNewTab->setDefaultAction(acnewTab);
	bookmarks = new QList<Bookmark*>();

	//Load bookmarks from file
	QFile f(QDir::homePath()+"/.lightweight2/bookmarks");
	if (f.exists()) {
		f.open(QFile::ReadOnly);
		QString line = QString::fromUtf8(f.readLine().data()).replace("\n", "");
		while (line != "END") {
			QString title = line;
			QString url = QString::fromUtf8(f.readLine().data()).replace("\n", "");
			QIcon icon(QDir::homePath()+"/.lightweight2/bookmarkicons/"+hashFor(url)+".png");

			addBookmark(title, url, icon);

			line = QString::fromUtf8(f.readLine().data()).replace("\n", "");
		}
		f.close();
	}

	connect(bookmarkMenu, SIGNAL(triggered(QAction*)), this, SLOT(openBookmark(QAction*)));
	connect(bookmarkMenu, SIGNAL(triggered(QAction*)), bookmarkMenu, SLOT(hide()));

	connect(ui->bBack, SIGNAL(clicked()), this, SLOT(back()));
	connect(ui->bForward, SIGNAL(clicked()), this, SLOT(forward()));
	connect(ui->bStop, SIGNAL(clicked()), this, SLOT(stop()));
	connect(ui->bReload, SIGNAL(clicked()), this, SLOT(reload()));

	connect(ui->tfSearch, SIGNAL(returnPressed()), this, SLOT(search()));

	//Create shortcuts
	new QShortcut(QKeySequence(tr("F11")), this, SLOT(switchFullscreen()));
	new QShortcut(QKeySequence(tr("Ctrl+Tab")), this, SLOT(nextTab()));
	new QShortcut(QKeySequence(tr("Ctrl+F4")), this, SLOT(closeCurrentTab()));

	//Default search engine
	ui->cbSearchEngine->addItem(QIcon(":/icons/search/google.png"), QString(), QVariant(tr("http://www.google.com/search?q=%{query}&ie=UTF-8&oe=UTF-8")));
	ui->cbSearchEngine->addItem(QIcon(":/icons/search/wikipedia.png"), QString(), QVariant(tr("http://en.wikipedia.org/wiki/%{query}")));

	//Get user search engines
	QDir dir(QDir::homePath()+"/.lightweight2/searchengines");
	QStringList subDirs = dir.entryList(QDir::Dirs, QDir::Name);
	for (int i = 0; i < subDirs.size(); i++) {
		QString subDirStr = (QString) subDirs.value(i);
		QFile file(QDir::homePath()+"/.lightweight2/searchengines/"+subDirStr+"/info");
		if (!file.exists()) continue;
		file.open(QFile::ReadOnly);
		QString url = file.readLine().replace("\n", "");
		ui->cbSearchEngine->addItem(QIcon(QDir::homePath()+QString("/.lightweight2/searchengines/%1/icon.png").arg(subDirStr)), QString(), QVariant(url));
		file.close();
	}

	//Set the last used search engine
	QSettings s;
	ui->cbSearchEngine->setCurrentIndex(s.value("searchengine", "0").toInt());

	//Create default tab
	newTab();
}

Lightweight::~Lightweight() {
	delete ui;
	while (tabList->size() > 0) {
		BrowserTab* tab = tabList->first();
		tabList->removeAll(tab);
		delete tab;
	}
	delete tabList;
}

void Lightweight::closeEvent(QCloseEvent *e) {
	//Save bookmarks to file
	QFile f(QDir::homePath()+"/.lightweight2/bookmarks");
	f.remove();
	f.open(QIODevice::ReadWrite);
	int i = 0;
	while (i < bookmarks->size()) {
		Bookmark* m = bookmarks->value(i);
		f.write((m->title()+"\n").toUtf8());
		f.write((m->url()+"\n").toUtf8());

		//Save icon to bookmark-file
		m->icon().pixmap(16,16).save(QDir::homePath()+"/.lightweight2/bookmarkicons/"+hashFor(m->url())+".png");
		i++;
	}
	f.write(QString("END").toUtf8());
	f.flush();
	f.close();

	QSettings s;
	s.setValue("searchengine", ui->cbSearchEngine->currentIndex());
	QWidget::closeEvent(e);
}

BrowserTab* Lightweight::newTab(QString url, bool hideUrlBar) {
	BrowserTab* tab = new BrowserTab();
	tabList->append(tab);
	ui->tabcontainer->layout()->addWidget(tab->tabButton);
	tabs->addWidget(tab->content);
	tabs->setCurrentWidget(tab->content);

	currentTab = tab;

	tab->goTo(url, hideUrlBar);
	connect(tab, SIGNAL(selected(BrowserTab*)), this, SLOT(showTab(BrowserTab*)));
	connect(tab, SIGNAL(closed(BrowserTab*)), this, SLOT(closeTab(BrowserTab*)));
	connect(tab, SIGNAL(newTab(QString)), this, SLOT(newTab(QString)));
	connect(tab, SIGNAL(bookmark(QString,QString,QIcon)), this, SLOT(addBookmark(QString,QString,QIcon)));
	connect(tab, SIGNAL(newWindow(QWebView**)), this, SLOT(newWindow(QWebView**)));

	return tab;
}

void Lightweight::newWindow(QWebView **view) {
	BrowserTab* t = newTab("about:blank", true);

	(*view) = t->getView();
}

void Lightweight::back() {
	currentTab->back();
}

void Lightweight::forward() {
	currentTab->forward();
}

void Lightweight::stop() {
	currentTab->stop();
}

void Lightweight::reload() {
	currentTab->reload();
}

void Lightweight::search() {
	QString queryTemplate = ui->cbSearchEngine->itemData(ui->cbSearchEngine->currentIndex()).toString();

	queryTemplate = queryTemplate.replace("%{query}", ui->tfSearch->text());
	if ((currentTab != 0) && currentTab->isEmptyPage()) currentTab->goTo(queryTemplate);
	else newTab(queryTemplate, true);
}

void Lightweight::showTab(BrowserTab *tab) {
	tabs->setCurrentWidget(tab->content);
	currentTab = tab;
}

void Lightweight::closeTab(BrowserTab *tab) {
	tabs->removeWidget(tab->content);
	ui->tabcontainer->layout()->removeWidget(tab->tabButton);

	tabList->removeAll(tab);

	if (!tabList->isEmpty() && (tab == currentTab)) {
		showTab(tabList->last());
	}

	if (tabList->isEmpty()) {
		currentTab = 0;
	}

	delete tab;
}

void Lightweight::openBookmark(QAction *ac) {
	Bookmark* b = (Bookmark*) ac;

	if (b->toBeDeleted()) {
		bookmarkMenu->removeAction(ac);
		bookmarks->removeAll(b);
		delete b;
	} else {
		if ((currentTab != 0) && currentTab->isEmptyPage()) currentTab->goTo(b->url());
		else newTab(b->url(), (b->url() != "about:blank"));
	}
}

void Lightweight::addBookmark(QString title, QString url, QIcon icon) {
	Bookmark* m = new Bookmark(icon, url, title);
	bookmarkMenu->addAction(m);
	bookmarks->append(m);
}

void Lightweight::switchFullscreen() {
	if (fullscreen) {
		this->setWindowState(Qt::WindowMaximized);
		fullscreen = false;
	} else {
		this->setWindowState(Qt::WindowFullScreen);
		fullscreen = true;
	}
}

void Lightweight::nextTab() {
	if (currentTab == 0) return;
	int i = (tabList->indexOf(currentTab)+1)%tabList->size();
	showTab(tabList->value(i));
}

void Lightweight::closeCurrentTab() {
	if (currentTab != 0) closeTab(currentTab);
}

QString Lightweight::hashFor(QString str) {
	QByteArray ar = QCryptographicHash::hash(str.toUtf8(), QCryptographicHash::Md5);
	return QString::fromUtf8(ar.toHex());
}
